local L = AceLibrary("AceLocale-2.2"):new("sRaidFrames")

function sRaidFrames:RegisterGroupSetup(name)
	self.opt.GroupSetups[name] = {}
	self:chatUpdateFilterMenu()
end

function sRaidFrames:GetGroupSetup(name)
	return self.opt.GroupSetups[name]
end

function sRaidFrames:RenameGroupSetup(old, new)
	self.opt.GroupSetups[new] = self.opt.GroupSetups[old]
	self.opt.GroupSetups[old] = nil
	if self.opt.GroupSetup == old then
		self.opt.GroupSetup = new
	end
	return self.opt.GroupSetups[new]
end

function sRaidFrames:DefaultGroupSetups()
	if not self:GetGroupSetup("By class") then
		self:RegisterGroupSetup("By class")
		self:RegisterGroupFilter("By class", 1, "Warriors", {["groupFilter"] = "WARRIOR"})
		self:RegisterGroupFilter("By class", 2, "Mages", {["groupFilter"] = "MAGE"})
		self:RegisterGroupFilter("By class", 3, "Paladins", {["groupFilter"] = "PALADIN"})
		self:RegisterGroupFilter("By class", 4, "Druids", {["groupFilter"] = "DRUID"})
		self:RegisterGroupFilter("By class", 5, "Hunters", {["groupFilter"] = "HUNTER"})
		self:RegisterGroupFilter("By class", 6, "Rogues", {["groupFilter"] = "ROGUE"})
		self:RegisterGroupFilter("By class", 7, "Warlocks", {["groupFilter"] = "WARLOCK"})
		self:RegisterGroupFilter("By class", 8, "Priests", {["groupFilter"] = "PRIEST"})
		self:RegisterGroupFilter("By class", 9, "Shamans", {["groupFilter"] = "SHAMAN"})
	end
	
	if not self:GetGroupSetup("By group") then
		self:RegisterGroupSetup("By group")
		self:RegisterGroupFilter("By group", 1, "Group 1", {["groupFilter"] = "1"})
		self:RegisterGroupFilter("By group", 2, "Group 2", {["groupFilter"] = "2"})
		self:RegisterGroupFilter("By group", 3, "Group 3", {["groupFilter"] = "3"})
		self:RegisterGroupFilter("By group", 4, "Group 4", {["groupFilter"] = "4"})
		self:RegisterGroupFilter("By group", 5, "Group 5", {["groupFilter"] = "5"})
		self:RegisterGroupFilter("By group", 6, "Group 6", {["groupFilter"] = "6"})
		self:RegisterGroupFilter("By group", 7, "Group 7", {["groupFilter"] = "7"})
		self:RegisterGroupFilter("By group", 8, "Group 8", {["groupFilter"] = "8"})
	end
	
	if not self:GetGroupSetup("By role") then
		self:RegisterGroupSetup("By role")
		self:RegisterGroupFilter("By role", 1, "Tanks", {["groupFilter"] = "WARRIOR"})
		self:RegisterGroupFilter("By role", 2, "Melee DPS", {["groupFilter"] = "ROGUE,WARRIOR"})
		self:RegisterGroupFilter("By role", 3, "Range DPS", {["groupFilter"] = "MAGE,WARLOCK,HUNTER"})
		self:RegisterGroupFilter("By role", 4, "Healers", {["groupFilter"] = "PALADIN,SHAMAN,PRIEST,DRUID"})
	end
	
	self:chatUpdateFilterMenu()
end

function sRaidFrames:DeleteGroupSetup(name)
	table.remove(self.opt.GroupSetups, name)
end

function sRaidFrames:GetGroupSetupFilter(name, group)
	return self.opt.GroupSetups[name][group]
end

function sRaidFrames:GetCurrentGroupSetup()
	return self:GetGroupSetup(self.opt.GroupSetup)
end

function sRaidFrames:RegisterGroupFilter(name, group, caption, attributes)
	if not attributes["sortMethod"] then
		attributes["sortMethod"] = "NAME"
	end
	if not attributes["sortDir"] then
		attributes["sortDir"] = "DESC"
	end
	
	self.opt.GroupSetups[name][group] = {["caption"] = caption, ["attributes"] = attributes}
	self:chatUpdateFilterMenu()
end

function sRaidFrames:SetCurrentGroupSetup(name)
	self:ScheduleLeaveCombatAction(function() self.opt.GroupSetup = name end)
	self:ScheduleLeaveCombatAction(self.CreateFrames, self)
	self:ScheduleLeaveCombatAction(self.SetGroupFilters, self)
	self:ScheduleLeaveCombatAction(self.Print, self, "Current filter set is now '".. name .."'")
	self:chatUpdateFilterMenu()
end

function sRaidFrames:GetAllGroupFilters()
	return self.opt.GroupSetups
end

function sRaidFrames:chatUpdateFilterMenu()
	local i = 1
	for name, _ in pairs(self:GetAllGroupFilters()) do
		self.options.args.sets.args["set".. i] = {}
		self.options.args.sets.args["set".. i].type = 'group'
		self.options.args.sets.args["set".. i].name = "Set: ".. name
		self.options.args.sets.args["set".. i].desc = name
		self.options.args.sets.args["set".. i].args = {}
		
		self.options.args.sets.args["set".. i].args["use"] = {}
		self.options.args.sets.args["set".. i].args["use"].type = 'execute'
		self.options.args.sets.args["set".. i].args["use"].name = "|cff00ff00Apply this set!"
		self.options.args.sets.args["set".. i].args["use"].desc = name
		self.options.args.sets.args["set".. i].args["use"].func = function() self:SetCurrentGroupSetup(name) self:chatUpdateFilterMenu() end
		self.options.args.sets.args["set".. i].args["use"].order = 1
		
		self.options.args.sets.args["set".. i].args["delete"] = {}
		self.options.args.sets.args["set".. i].args["delete"].type = 'execute'
		self.options.args.sets.args["set".. i].args["delete"].name = "|cffff0000Delete this set"
		self.options.args.sets.args["set".. i].args["delete"].desc = name
		self.options.args.sets.args["set".. i].args["delete"].func = function() self:DeleteGroupSetup(name) self:chatUpdateFilterMenu() end
		self.options.args.sets.args["set".. i].args["delete"].order = 300
		
		self.options.args.sets.args["set".. i].args["rename"] = {}
		self.options.args.sets.args["set".. i].args["rename"].type = 'text'
		self.options.args.sets.args["set".. i].args["rename"].name = "Rename this set"
		self.options.args.sets.args["set".. i].args["rename"].desc = name
		self.options.args.sets.args["set".. i].args["rename"].get = function() return name end
		self.options.args.sets.args["set".. i].args["rename"].set = function(value) 
																													self:RenameGroupSetup(name, value)
																													self:chatUpdateFilterMenu() 
																													end
		self.options.args.sets.args["set".. i].args["rename"].usage = "<Name>"
		self.options.args.sets.args["set".. i].args["rename"].order = 2
		
		self.options.args.sets.args["set".. i].args["spacer1"] = {}
		self.options.args.sets.args["set".. i].args["spacer1"].type = 'header'
		self.options.args.sets.args["set".. i].args["spacer1"].order = 3
		
		self.options.args.sets.args["set".. i].args["spacer2"] = {}
		self.options.args.sets.args["set".. i].args["spacer2"].type = 'header'
		self.options.args.sets.args["set".. i].args["spacer2"].order = 200
		
		for id, data in pairs(self:GetGroupSetup(name)) do
			self.options.args.sets.args["set".. i].args["frame".. id] = {}
			self.options.args.sets.args["set".. i].args["frame".. id].type = 'group'
			self.options.args.sets.args["set".. i].args["frame".. id].name = "Frame #".. id .. " (".. data.caption ..")"
			self.options.args.sets.args["set".. i].args["frame".. id].desc = "Frame ".. id
			self.options.args.sets.args["set".. i].args["frame".. id].args = {}
			
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"] = {}
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"].type = 'text'
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"].name = 'Caption'
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"].desc = 'Caption'
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"].get = function() return data.caption end
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"].set = function(value) 	
																																												self:RegisterGroupFilter(name, id, value, data.attributes)
																																												self:chatUpdateFilterMenu() 
																																												self:SetGroupFilters()
																																											end
			self.options.args.sets.args["set".. i].args["frame".. id].args["caption"].usage = "<Caption>"
			
			self.options.args.sets.args["set".. i].args["frame".. id].args["classes"] = {}
			self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].type = 'group'
			self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].name = 'Classes'
			self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].desc = 'Classes'
			self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args = {}
			
			for eclass, class in pairs({["WARRIOR"] = L["Warriors"], ["PALADIN"] = L["Paladins"], ["SHAMAN"] = L["Shamans"], ["PRIEST"] = L["Priests"], ["ROGUE"] = L["Rogues"], ["HUNTER"] = L["Hunters"], ["MAGE"] = L["Mages"], ["WARLOCK"] = L["Warlocks"], ["DRUID"] = L["Druids"]}) do
				self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args[eclass] = {}
				self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args[eclass].type = 'toggle'
				self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args[eclass].name = class
				self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args[eclass].desc = class
				self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args[eclass].get = function() 
																																																			local groupFilter = explode(",", data.attributes.groupFilter) 
																																																			for _, _class in pairs(groupFilter) do
																																																				if _class == eclass then
																																																					return true
																																																				end
																																																			end
																																																			return false
																																																		end
				self.options.args.sets.args["set".. i].args["frame".. id].args["classes"].args[eclass].set = function(value) 
																																																			local groupFilter = explode(",", data.attributes.groupFilter) 
																																																			if value then
																																																				table.insert(groupFilter, eclass)
																																																			else
																																																				classes = table_removeByValue(groupFilter, eclass)
																																																			end
																																																			
																																																			data.attributes.groupFilter = implode(",", groupFilter)
																																																			self:RegisterGroupFilter(name, id, data.caption, data.attributes)
																																																			self:chatUpdateFilterMenu() 
																																																			self:SetGroupFilters()
																																																		 end			
			end
			self.options.args.sets.args["set".. i].args["frame".. id].args["groups"] = {}
			self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].type = 'group'
			self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].name = 'Groups'
			self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].desc = 'Groups'
			self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args = {}
			
			for group = 1, 8 do
				self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args["group".. group] = {}
				self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args["group".. group].type = 'toggle'
				self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args["group".. group].name = "Group ".. group
				self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args["group".. group].desc = "Group ".. group
				self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args["group".. group].get = function() 
																																																			local groupFilter = explode(",", data.attributes.groupFilter) 
																																																			for _, _group in pairs(groupFilter) do
																																																				if tonumber(_group) == group then
																																																					self:Print("MATCH")
																																																					return true
																																																				end
																																																			end
																																																			return false
																																																		end
				self.options.args.sets.args["set".. i].args["frame".. id].args["groups"].args["group".. group].set = function(value) 
																																																			local groupFilter = explode(",", data.attributes.groupFilter) 
																																																			if value then
																																																				table.insert(groupFilter, group)
																																																			else
																																																				groupFilter = table_removeByValue(groupFilter, group)
																																																			end
																																																			
																																																			data.attributes.groupFilter = implode(",", groupFilter)
																																																			self:RegisterGroupFilter(name, id, data.caption, data.attributes)
																																																			self:chatUpdateFilterMenu() 
																																																			self:SetGroupFilters()
																																																		 end	
			end
		end
		i = i + 1
	end
end


function explode(div,str)
	local pos,arr = 0,{}
	for st,sp in function() return string.find(str,div,pos,true) end do -- for each divider found
		table.insert(arr,string.sub(str,pos,st-1)) -- Attach chars left of current divider
		pos = sp + 1 -- Jump past current divider
	end
	table.insert(arr,string.sub(str,pos)) -- Attach chars right of last divider
	return arr
end

function implode(glue, array)
	return table.concat(array, glue)
end

function table_removeByValue(t, value)
	for k,v in pairs(t) do
		if v == value then
			table.remove(t, k)
		end
	end
	return t
end